@@ -2,6 +2,8 @@
 
 /*
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2003 Kim Woelders
+ * Copyright (C) 2003 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -42,7 +44,8 @@ static GHashTable *window_hash = NULL;
     ((window)->priv->skip_pager          << 4) |        \
     ((window)->priv->skip_taskbar        << 5) |        \
     ((window)->priv->is_sticky           << 6) |        \
-    ((window)->priv->is_hidden           << 7) )
+    ((window)->priv->is_hidden           << 7) |        \
+    ((window)->priv->is_fullscreen       << 8) )
 
 struct _WnckWindowPrivate
 {
@@ -91,6 +94,7 @@ struct _WnckWindowPrivate
   guint skip_taskbar : 1;
   guint is_sticky : 1;
   guint is_hidden : 1;
+  guint is_fullscreen : 1;
 
   /* _NET_WM_STATE_HIDDEN doesn't map directly into an
    * externally-visible state (it determines the WM_STATE
@@ -679,6 +683,14 @@ wnck_window_is_skip_tasklist          (WnckWindow *window)
   return window->priv->skip_taskbar;
 }
 
+gboolean
+wnck_window_is_fullscreen                 (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
+
+  return window->priv->is_fullscreen;
+}
+
 void
 wnck_window_set_skip_tasklist (WnckWindow *window,
                                gboolean skip)
@@ -691,6 +703,18 @@ wnck_window_set_skip_tasklist (WnckWindow *window,
                       0);
 }
 
+void
+wnck_window_set_fullscreen (WnckWindow *window,
+                               gboolean fullscreen)
+{
+  g_return_if_fail (WNCK_IS_WINDOW (window));
+  _wnck_change_state (WNCK_SCREEN_XSCREEN (window->priv->screen),
+		      window->priv->xwindow,
+                      fullscreen,
+                      _wnck_atom_get ("_NET_WM_STATE_FULLSCREEN"),
+                      0);
+}
+
 /**
  * wnck_window_is_sticky:
  * @window: a #WnckWindow
@@ -1284,6 +1308,45 @@ wnck_window_is_on_workspace (WnckWindow    *window,
     wnck_window_get_workspace (window) == workspace;
 }
 
+/**
+ * wnck_window_is_in_viewport:
+ * @window: a #WnckWindow
+ * @workspace: a #WnckWorkspace
+ * 
+ * Returns #TRUE if the window is inside the current viewport
+ * of the given workspace.
+ * 
+ * Return value: %TRUE if @window appears in current viewport
+ **/
+gboolean
+wnck_window_is_in_viewport (WnckWindow    *window,
+                            WnckWorkspace *workspace)
+{
+  GdkRectangle window_rect;
+  GdkRectangle viewport_rect;
+  
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
+  g_return_val_if_fail (WNCK_IS_WORKSPACE (workspace), FALSE);
+
+  if (wnck_window_is_pinned (window) )
+    return TRUE;
+
+  if (wnck_window_get_workspace (window) != workspace)
+    return FALSE;
+
+  viewport_rect.x = wnck_workspace_get_viewport_x (workspace);
+  viewport_rect.y = wnck_workspace_get_viewport_y (workspace);
+  viewport_rect.width = wnck_screen_get_width (window->priv->screen);
+  viewport_rect.height = wnck_screen_get_height (window->priv->screen);
+
+  window_rect.x = window->priv->x + viewport_rect.x;
+  window_rect.y = window->priv->y + viewport_rect.y;
+  window_rect.width = window->priv->width;
+  window_rect.height = window->priv->height;
+
+  return gdk_rectangle_intersect (&viewport_rect, &window_rect, &window_rect);
+}
+
 void
 _wnck_window_set_application (WnckWindow      *window,
                               WnckApplication *app)
@@ -1472,6 +1535,8 @@ update_state (WnckWindow *window)
             window->priv->is_sticky = TRUE;
           else if (atoms[i] == _wnck_atom_get ("_NET_WM_STATE_SHADED"))
             window->priv->is_shaded = TRUE;
+          else if (atoms[i] == _wnck_atom_get ("_NET_WM_STATE_FULLSCREEN"))
+            window->priv->is_fullscreen = TRUE;
           else if (atoms[i] == _wnck_atom_get ("_NET_WM_STATE_SKIP_TASKBAR"))
             window->priv->skip_taskbar = TRUE;
           else if (atoms[i] == _wnck_atom_get ("_NET_WM_STATE_SKIP_PAGER"))
@@ -1851,12 +1916,9 @@ force_update_now (WnckWindow *window)
    */
 
   old_name = window->priv->name;
-  old_icon_name = window->priv->icon_name;
   window->priv->name = NULL;
-  window->priv->icon_name = NULL;
 
   update_name (window);
-  update_icon_name (window);
 
   if (window->priv->name == NULL)
     window->priv->name = old_name;
@@ -1867,6 +1929,11 @@ force_update_now (WnckWindow *window)
       g_free (old_name);
     }
 
+  old_icon_name = window->priv->icon_name;
+  window->priv->icon_name = NULL;
+
+  update_icon_name (window);
+
   if (window->priv->icon_name == NULL)
     window->priv->icon_name = old_icon_name;
   else
